"""awswrangler catalog utilities."""
from typing import Any

import awswrangler as wr
import pandas as pd


def get_schema_df(**kwargs: Any) -> pd.DataFrame:
    """Return a Pandas dataframe having the combined schema of each of the tables in the AWS Catalog.

    The returned dataframe has the columns: Database, Table, Column Name, Type, Partition, Comment

    :param kwargs: These are forwarded to `awswrangler.catalog.tables` for filtering the tables.
                   The `catalog_id` and `boto3_session` args are also forwarded to awswrangler.catalog.table.

    Example:
    -------
        df = get_schema_df(database="my_db", name_prefix="my_tables_")

    """
    # Ref: https://aws-data-wrangler.readthedocs.io/en/stable/stubs/awswrangler.catalog.tables.html#awswrangler.catalog.tables
    def get_table_df(database: str, table: str) -> pd.DataFrame:
        df_table = wr.catalog.table(database=database, table=table, catalog_id=kwargs.get("catalog_id"), boto3_session=kwargs.get("boto3_session"))
        assert all(c not in df_table.columns for c in ("Database", "Table"))
        df_table.insert(0, "Database", database)
        df_table.insert(1, "Table", table)
        return df_table

    df_tables = wr.catalog.tables(**kwargs)
    if df_tables.empty:
        output_dtypes = {"Database": str, "Table": str, "Column Name": str, "Type": str, "Partition": bool, "Comment": str}
        return pd.DataFrame({c: pd.Series(dtype=t) for c, t in output_dtypes.items()})
    return pd.concat([get_table_df(t.Database, t.Table) for t in wr.catalog.tables(**kwargs).itertuples()], ignore_index=True)



"""Presto SQL utilities."""
from typing import List


def date_parse(formats: List[str], /, column: str = "{col}") -> str:
    """Return the coalesced date_parse usage expression to parse a column using the given formats.

    Example:
    -------
        date_parse(['%m/%d/%Y %r', '%Y-%m-%d %H:%i:%s', '%Y-%m-%d %H:%i:%s.%f']) ->
            "coalesce(try(date_parse({col}, '%m/%d/%Y %r')), try(date_parse({col}, '%Y-%m-%d %H:%i:%s')), date_parse({col}, '%Y-%m-%d %H:%i:%s.%f'))"

    """
    assert formats

    def _parse(fmt: str, /) -> str:
        return f"date_parse({column}, '{fmt}')"

    if len(formats) == 1:
        return _parse(formats[0])

    return "coalesce(" + ", ".join(f"try({_parse(f)})" for f in formats[:-1]) + ", " + _parse(formats[-1]) + ")"


def base64_hash_id(columns: List[str], /, *, num_bytes: int) -> str:
    """Return an expression of a base64 hash ID corresponding to the given columns having the given length in bytes.

    Example:
    -------
        base64_hash_id(['foo', 'bar', 'coalesce(baz, qux)'], num_bytes=8) -> "to_base64url(substr(sha256(to_utf8(foo || '&' || bar || '&' || coalesce(baz, qux))), -8))"

    """
    assert columns
    assert 1 <= num_bytes <= 32  # This is compatible with sha256.
    return "to_base64url(substr(sha256(to_utf8(" + " || '&' || ".join(columns) + f")), -{num_bytes}))"



"""textwrap utilities."""
import textwrap


def indent4(text: str, *, levels: int) -> str:
    """Indent the given string by the specified number of levels."""
    return textwrap.indent(text, prefix=" " * 4 * levels)
